"use strict";
/**
 * @hidden
 * @ignore
 * @internal
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestClient = exports.STATION_GROUP_ID_NOT_EXISTS = void 0;
/**
 * This comment and the above tags will tell typedoc to ignore the whole module.
 */
const tough_cookie_1 = require("tough-cookie");
const Constants_1 = require("./Constants");
const models_1 = require("./models");
exports.STATION_GROUP_ID_NOT_EXISTS = -1;
const axios = require('axios').default;
const axiosCookieJarSupport = require('@3846masa/axios-cookiejar-support')
    .default;
const logger = Constants_1.log4js.getLogger('RestClient');
axios.interceptors.request.use((request) => {
    logger.debug('Starting Request', JSON.stringify(request, null, 2));
    return request;
});
axiosCookieJarSupport(axios);
class RestClient {
    constructor(baseUrl, masterToken) {
        this.baseUrl = baseUrl;
        const masterCookieJar = new tough_cookie_1.CookieJar();
        this.masterCredential = {
            token: `Bearer ${masterToken}`,
            cookieJar: masterCookieJar
        };
        this.credentials = new Map();
    }
    getUser() {
        const url = `${this.baseUrl}/user`;
        const options = this.prepareGetOptions(url);
        return axios(options).then((response) => response.data);
    }
    getUserToken(username) {
        const url = `${this.baseUrl}/user/${username}/token`;
        const options = this.prepareGetOptions(url);
        return axios(options).then((response) => {
            return response.data;
        });
    }
    storeUserToken(username, userToken) {
        const credential = { token: userToken, cookieJar: new tough_cookie_1.CookieJar() };
        this.credentials.set(username, credential);
    }
    getAndStoreUserToken(username) {
        return __awaiter(this, void 0, void 0, function* () {
            const userToken = yield this.getUserToken(username);
            logger.debug(`storing token for ${username}`);
            this.storeUserToken(username, userToken);
        });
    }
    /**
     * Caller should await the method to finish. When call is successful, a number greater than 0 will be returned.
     * When subAccountId has no station group defined, -1 will be returned;
     * For other errors, a negative value of http status code will be returned;
     * @param subAccountId
     */
    getAgentStationGroupId(subAccountAppId) {
        const stationGroupUri = Constants_1.STATION_GROUP_PATH.replace('{subAccountAppId}', subAccountAppId);
        const url = `${this.baseUrl}/${stationGroupUri}`;
        logger.debug(`getAgentStationGroupId url is ${url}`);
        const options = this.prepareGetOptions(url);
        return axios(options)
            .then((response) => {
            let agentStationGroups = response.data;
            if (agentStationGroups.length === 0) {
                return exports.STATION_GROUP_ID_NOT_EXISTS;
            }
            // ensure we always get the same subAccount ordering
            agentStationGroups = Constants_1.lodash.sortBy(agentStationGroups, ['id']);
            return agentStationGroups[0].id;
        })
            .catch((error) => {
            return -error.response.status;
        });
    }
    getSubAccount() {
        const url = `${this.baseUrl}/${Constants_1.USER_PATH}`;
        logger.debug(`getSubAccount url is ${url}`);
        const options = this.prepareGetOptions(url);
        return axios(options).then((response) => {
            let accessibleSubAccounts = response.data.accessibleClients;
            accessibleSubAccounts = Constants_1.lodash.sortBy(accessibleSubAccounts, ['id']);
            const subAccount = accessibleSubAccounts[0];
            return subAccount;
        });
    }
    getSubAccountId() {
        return __awaiter(this, void 0, void 0, function* () {
            const id = yield this.getSubAccount().then((response) => {
                return response.id;
            });
            logger.debug(`Sub Account App ID is: ${id}`);
            return id;
        });
    }
    getSubAccountAppId() {
        return __awaiter(this, void 0, void 0, function* () {
            const appId = yield this.getSubAccount().then((response) => {
                return response.appId;
            });
            return appId;
        });
    }
    getSubAccountIdAndAppId() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.getSubAccount().then((response) => {
                return { appId: response.appId, id: response.id };
            });
            return result;
        });
    }
    getAgentByUsername(agentUsername) {
        const url = `${this.baseUrl}/${Constants_1.FETCH_AGENT_BY_USERNAME_PATH}/${agentUsername}`;
        logger.debug(`getAgentByUsername url is ${url}`);
        const options = this.prepareGetOptions(url);
        return axios(options).then((response) => {
            // logger.debug(response.data)
            return response.data;
        });
    }
    /**
     * return true if agent deletion is requested successfully
     * return false otherwise
     * @param agentUsername
     * @param agentLoginId
     */
    requestAgentDeletion(agentUsername, agentLoginId) {
        const url = `${this.baseUrl}/${Constants_1.REMOVE_AGENT_PATH}`;
        const deleteRequest = { username: agentUsername, loginId: agentLoginId };
        const options = this.prepareBaseOptions();
        return axios
            .post(url, deleteRequest, options)
            .then((response) => {
            return true;
        })
            .catch((error) => {
            logger.debug(error.response.status);
            return false;
        });
    }
    prepareBaseOptions() {
        const cookieJar = this.masterCredential.cookieJar;
        return {
            headers: { Authorization: this.masterCredential.token },
            jar: cookieJar,
            withCredentials: true
        };
    }
    prepareGetOptions(url) {
        return Object.assign(Object.assign({}, this.prepareBaseOptions()), { url, method: 'GET' });
    }
    preparePostOptions(url) {
        return Object.assign(Object.assign({}, this.prepareBaseOptions()), { url, method: 'POST' });
    }
    prepareDeleteOptions(url) {
        return Object.assign(Object.assign({}, this.prepareBaseOptions()), { url, method: 'DELETE' });
    }
    /**
     * return agent or undefined
     * @param loginId
     */
    getAgentByLoginId(loginId) {
        const url = `${this.baseUrl}/${Constants_1.FETCH_AGENT_ID_PATH}/${loginId}`;
        const options = this.prepareGetOptions(url);
        return axios(options).then((response) => {
            return response.data;
        });
    }
    createAgentJob(agent) {
        const url = `${this.baseUrl}/${Constants_1.AGENT_JOB_PATH}`;
        const options = this.prepareBaseOptions();
        return axios.post(url, agent, options).then((result) => {
            return result;
        });
    }
    /**
     * return extension number if successful.otherwise a negative status code
     * @param subAccountId
     * @param type
     */
    getNextAvailableExtension(subAccountId, type) {
        const url = `${this.baseUrl}/${Constants_1.EXTENSION_PATH}/${subAccountId}/type/${type}`;
        return this.postToUrl(url);
    }
    getNextAvailableNumber(subAccountId, type) {
        const url = `${this.baseUrl}/${Constants_1.NUMBER_PATH}/${subAccountId}/type/${type}`;
        return this.postToUrl(url);
    }
    createSkillV2(skill) {
        const url = `${this.baseUrl}/${Constants_1.SKILLV2_PATH}`;
        logger.debug(`createSkillV2 url = ${url}`);
        const options = this.prepareBaseOptions();
        return axios
            .post(url, skill, options)
            .then((result) => {
            // logger.debug(result.data)
            return result.status;
        })
            .catch((error) => {
            logger.debug(error.response.status);
            return -error.response.status;
        });
    }
    getSubAccountAgentSkills(subAccountAppId) {
        const url = `${this.baseUrl}/${Constants_1.FETCH_SKILL_ID_PATH}${subAccountAppId}`;
        logger.debug(url);
        const options = this.prepareGetOptions(url);
        return axios(options).then((response) => {
            return response;
        });
    }
    getSkillV2(skillNumber) {
        const url = `${this.baseUrl}/${Constants_1.SKILLV2_PATH}/${skillNumber}`;
        logger.debug(url);
        const options = this.prepareGetOptions(url);
        return axios(options).then((response) => {
            return response;
        });
    }
    makeSubAccountSubscriptionUrl(subAccountAppId) {
        const path = `${this.baseUrl}/${Constants_1.SUBSCRIPTION_PATH}`;
        if (subAccountAppId === undefined) {
            return path;
        }
        else {
            return `${path}?subAccountAppId=${subAccountAppId}`;
        }
    }
    /**
     * create data subscription given a valid subAccountAppId and request.  Returns a subscription response on success
     * or empty object on error
     *
     * @param createSubscriptionRequest
     */
    createDataSubscription(createSubscriptionRequest) {
        const url = this.makeSubAccountSubscriptionUrl(undefined);
        logger.debug(`createDataSubscription url = ${url}`);
        const options = this.prepareBaseOptions();
        return axios
            .post(url, createSubscriptionRequest, options)
            .then((response) => response.data)
            .catch((error) => {
            logger.debug(error.response.status);
            return {};
        });
    }
    getAllSubscriptions(subAccountAppId) {
        const url = this.makeSubAccountSubscriptionUrl(subAccountAppId);
        logger.debug(`getAllSubscriptions url = ${url}`);
        const options = this.prepareGetOptions(url);
        return axios(options).then((response) => response.data);
    }
    /**
     *
     * @param subAccountAppId
     * @param subscriptionId
     * @param updateSubscriptionRequest
     */
    updateDataSubscription(subAccountAppId, subscriptionId, updateSubscriptionRequest) {
        const url = this.makeSubscriptionUrl(subAccountAppId, subscriptionId);
        const options = this.prepareBaseOptions();
        return axios
            .put(url, updateSubscriptionRequest, options)
            .then((response) => response.data)
            .catch((error) => error.response.status);
    }
    /**
     * return status code when success or negative status code in error
     * @param subAccountAppId
     * @param subscriptionId usually it is a uuid
     */
    deleteDataSubscription(subAccountAppId, subscriptionId) {
        const url = this.makeSubscriptionUrl(subAccountAppId, subscriptionId);
        logger.debug(`deleteDataSubscription url = ${url}`);
        const options = this.prepareDeleteOptions(url);
        return axios(options)
            .then((response) => response.status)
            .catch((error) => {
            logger.error(error);
            return -error.response.status;
        });
    }
    makeSubscriptionUrl(subAccountAppId, subscriptionId) {
        return `${this.baseUrl}/${Constants_1.SUBSCRIPTION_PATH}/${subscriptionId}?${Constants_1.SUB_ACCOUNT_KEY}=${subAccountAppId}`;
    }
    /**
     * get Subscription by id
     * @param subAccountAppId
     * @param subscriptionId
     */
    getDataSubscription(subAccountAppId, subscriptionId) {
        const url = this.makeSubscriptionUrl(subAccountAppId, subscriptionId);
        logger.debug(url);
        const options = this.prepareGetOptions(url);
        return axios(options)
            .then((response) => response.data)
            .catch((error) => {
            logger.debug(error.response.status);
            return {};
        });
    }
    printMasterCookieJar() {
        return JSON.stringify(this.masterCredential.cookieJar);
    }
    getAuxCodesBySubaccountId(subAccountId) {
        const url = `${this.baseUrl}/${Constants_1.FETCH_AUXCODE_BASE}/${subAccountId}/${Constants_1.FETCH_AUX_CODES}`;
        const options = this.prepareGetOptions(url);
        return axios(options).then((response) => {
            return response.data;
        });
    }
    getEffectiveAuxCodesBySubaccountId(subAccountId) {
        const url = `${this.baseUrl}/${Constants_1.FETCH_AUXCODE_BASE}/${subAccountId}/${Constants_1.FETCH_EFFECTIVE_AUX_CODES}`;
        const options = this.prepareGetOptions(url);
        return axios(options).then((response) => {
            return response.data;
        });
    }
    getAUXCodeForEffectiveAppId() {
        return __awaiter(this, void 0, void 0, function* () {
            const subAccountAppId = yield this.getSubAccountAppId();
            const url = `${this.baseUrl}/${Constants_1.FETCH_AUXCODE_BASE}/${subAccountAppId}/${Constants_1.FETCH_AUX_CODE_WITH_SUBACCOUNT_APP_ID}`;
            const options = this.prepareGetOptions(url);
            return axios(options).then((response) => {
                return response.data;
            });
        });
    }
    getAddressBook() {
        return __awaiter(this, void 0, void 0, function* () {
            const subAccountAppId = yield this.getSubAccountAppId();
            const url = `${this.baseUrl}/${Constants_1.ADDRESS_BOOK_PATH}/${subAccountAppId}`;
            const options = this.prepareGetOptions(url);
            return axios(options).then((response) => {
                return response.data;
            });
        });
    }
    searchContacts(searchRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            const subAccountAppId = yield this.getSubAccountAppId();
            const url = `${this.baseUrl}/${Constants_1.ADDRESS_BOOK_PATH}/search/${subAccountAppId}`;
            const queryParams = models_1.buildQueryParams(searchRequest);
            const options = this.prepareGetOptions(url + queryParams);
            return axios(options).then((response) => {
                return response.data;
            });
        });
    }
    postToUrl(url) {
        logger.debug(`url=${url}`);
        const options = this.preparePostOptions(url);
        return axios(options)
            .then((response) => {
            return response.data;
        })
            .catch((error) => {
            logger.debug(error.response.status);
            return -error.response.status;
        });
    }
}
exports.RestClient = RestClient;
//# sourceMappingURL=RestClient.js.map