"use strict";
/**
 * @hidden
 * @ignore
 * @internal
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isValidUrl = exports.isValidLocalUrl = exports.isTokenWellFormed = exports.isEmpty = exports.isValidFirstLastName = exports.isValidUsername = exports.hasAllowableCharacters = exports.hasWhiteSpace = exports.hasSpecialCharacter = exports.hasUpperCase = exports.hasLowerCase = exports.isValidPassword = exports.getValue = exports.removeSingleQuote = exports.sleep = exports.isValidSkillsWithPriorities = exports.randomString = exports.skillDecoder = void 0;
/**
 * This comment and the above tags will tell typedoc to ignore the whole module.
 */
const ts_data_json_1 = require("ts.data.json");
const Constants_1 = require("./Constants");
const logger = Constants_1.log4js.getLogger('Utils');
const URL = require('url').URL;
// Username in mpact.user and ac_station.user is 50 chars long; Bulk upload uses 49, so does here.
const MAX_USERNAME_LENGTH = 49;
const MAX_FIRST_LAST_NAME_LENGTH = 16;
function isValidParameter(key, parameter) {
    if (parameter === undefined) {
        logger.error(key + ' was undefined');
        return false;
    }
    return true;
}
exports.default = isValidParameter;
exports.skillDecoder = ts_data_json_1.JsonDecoder.object({
    skillNumber: ts_data_json_1.JsonDecoder.number,
    skillPriority: ts_data_json_1.JsonDecoder.number
}, 'SkillPriority');
function randomString(length) {
    const inOptions = 'abcdefghijklmnopqrstuvwxyz0123456789';
    let outString = inOptions.charAt(Math.floor(Math.random() * 26));
    for (let i = 1; i < length; i++) {
        outString += inOptions.charAt(Math.floor(Math.random() * inOptions.length));
    }
    return outString;
}
exports.randomString = randomString;
function isValidSkillsWithPriorities(key, skillPriorities) {
    const obj = JSON.parse(skillPriorities);
    for (const skill of obj) {
        if (exports.skillDecoder.decode(skill) instanceof ts_data_json_1.Err) {
            return false;
        }
    }
    return true;
}
exports.isValidSkillsWithPriorities = isValidSkillsWithPriorities;
/**
 * sleep for millis seconds must be called with await
 * @param ms
 */
function sleep(ms) {
    return __awaiter(this, void 0, void 0, function* () {
        yield new Promise(resolve => setTimeout(resolve, ms));
    });
}
exports.sleep = sleep;
function removeSingleQuote(s) {
    return s.replace(Constants_1.REPLACE_REGEX, Constants_1.EMPTY_STRING);
}
exports.removeSingleQuote = removeSingleQuote;
function getValue(key, args) {
    const value = args[key];
    if (isValidParameter(key, value)) {
        return removeSingleQuote(value);
    }
    else {
        throw new Error(`${key} needs to be specified`);
    }
}
exports.getValue = getValue;
/**
 * check if password is valid
 * @param password, min length 8, max length 32, must have a uppercase character, must have at least one lowercase char, no whitespace, must contains a number, must contain one of ~!@?#$%^&*_
 */
function isValidPassword(password) {
    if (password.length < 8 || password.length > 32) {
        return false;
    }
    if (!hasLowerCase(password)) {
        return false;
    }
    if (!hasUpperCase(password)) {
        return false;
    }
    if (!hasSpecialCharacter(password)) {
        return false;
    }
    if (hasWhiteSpace(password)) {
        return false;
    }
    return true;
}
exports.isValidPassword = isValidPassword;
function hasLowerCase(str) {
    return /[a-z]/.test(str);
}
exports.hasLowerCase = hasLowerCase;
function hasUpperCase(str) {
    return /[A-Z]/.test(str);
}
exports.hasUpperCase = hasUpperCase;
function hasSpecialCharacter(str) {
    return /[~!@?#$%^&*_]/.test(str);
}
exports.hasSpecialCharacter = hasSpecialCharacter;
function hasWhiteSpace(str) {
    return /\s/g.test(str);
}
exports.hasWhiteSpace = hasWhiteSpace;
function hasAllowableCharacters(str) {
    return /^[-.@\w]+$/g.test(str);
}
exports.hasAllowableCharacters = hasAllowableCharacters;
/**
 * check if a username is valid
 * @param username min length 2, max length 49, must pass ^[-.@\w]+$
 */
function isValidUsername(username) {
    if (username.length < 2 || username.length > MAX_USERNAME_LENGTH) {
        return false;
    }
    return hasAllowableCharacters(username);
}
exports.isValidUsername = isValidUsername;
/**
 * check if first or last name is valid
 * @param name min length 2, max length 16, must pass /^[a-zA-Z][a-zA-Z0-9_ .,'+]*$/
 */
function isValidFirstLastName(name) {
    if (name.length < 2 || name.length > MAX_FIRST_LAST_NAME_LENGTH) {
        return false;
    }
    return /^[a-zA-Z][a-zA-Z0-9_ .,'+]*$/.test(name);
}
exports.isValidFirstLastName = isValidFirstLastName;
function isEmpty(data) {
    if (typeof data === 'object') {
        if (JSON.stringify(data) === '{}' || JSON.stringify(data) === '[]') {
            return true;
        }
        else if (!data) {
            return true;
        }
        return false;
    }
    else if (typeof data === 'string') {
        if (!data.trim()) {
            return true;
        }
        return false;
    }
    else if (typeof data === 'undefined') {
        return true;
    }
    else {
        return false;
    }
}
exports.isEmpty = isEmpty;
function isTokenWellFormed(token) {
    if (isEmpty(token)) {
        return false;
    }
    try {
        const payload = Constants_1.jwt.decode(token);
        return !isEmpty(payload);
    }
    catch (error) {
        logger.warn(error);
        return false;
    }
}
exports.isTokenWellFormed = isTokenWellFormed;
function isValidLocalUrl(url) {
    const regex = /^https?:\/\/localhost:\d{2,5}$/;
    return regex.test(url);
}
exports.isValidLocalUrl = isValidLocalUrl;
function isValidUrl(url) {
    if (isEmpty(url)) {
        return false;
    }
    if (url.includes('localhost')) {
        return isValidLocalUrl(url);
    }
    try {
        new URL(url);
        return true;
    }
    catch (error) {
        logger.warn(error);
        return false;
    }
}
exports.isValidUrl = isValidUrl;
//# sourceMappingURL=Utils.js.map